
package com.dotfile.palm.config;

import java.io.*;
import palm.conduit.Log;


public
abstract
class		HotSyncConfig
implements	HotSyncConstants
	{
	/**
	 * The synchronization action to be taken.
	 */
	protected short		syncAction;

	/**
	 * The config directory in which our conduit lives.
	 */
	protected File		configDir;


	/**
	 * Returns the name of the subdirectory where
	 * we to store our settings the other files.
	 */

	abstract public String
		getPropertyDirName();

	/**
	 * Returns the name of the File we use to store our settings.
	 */

	abstract public String
		getPropertyFileName();

	/**
	 * Initializes the config's specific fields before reading
	 * them from the config file.
	 */

	abstract public void
		loadDefaultConfig();

	abstract public void
		loadConduitConfig( DataInputStream dataIn )
			throws IOException;

	abstract public void
		saveConduitConfig( DataOutputStream dataOut )
			throws IOException;

	public
	HotSyncConfig( String configPath )
		{
		this.syncAction = HOTSYNC_ACTION_SYNC;
		this.setConfigDirectory( configPath );
		this.loadDefaultConfig();
		this.loadConfiguration();
		}

	/**
	 * Returns the File we use to store our settings.
	 */

	public File
	getPropertiesFile()
		{
		return new File( this.configDir, this.getPropertyFileName() );
		}

	public File
	getConfigDirectory()
		{
		return this.configDir;
		}

	private void
	setConfigDirectory( String configPath )
		{
		if ( configPath.endsWith( File.separator ) )
			{
			configPath = configPath.substring
				( 0, (configPath.length()
						- File.separator.length()) );
			}

		String name = configPath;
		int idx = configPath.lastIndexOf( File.separator );
		if ( idx >= 0 )
			{
			name = configPath.substring( idx + 1 );
			}

		this.configDir = new File( configPath );

		String prefDir = this.getPropertyDirName();

		if ( ! name.equalsIgnoreCase( prefDir ) )
			{
			this.configDir =
				new File( this.configDir, prefDir );
			}

		if ( ! this.configDir.exists() )
			{
			this.configDir.mkdirs();
			}
		}

	public short
	getSyncAction()
		{
		return this.syncAction;
		}

	public void
	setSyncAction( short syncAction )
		{
		this.syncAction = syncAction;
		}

	public boolean
	isActionSync()
		{
		return (this.syncAction == HOTSYNC_ACTION_SYNC);
		}

	public boolean
	isActionHH2PC()
		{
		return (this.syncAction == HOTSYNC_ACTION_HH2PC);
		}

	public boolean
	isActionPC2HH()
		{
		return (this.syncAction == HOTSYNC_ACTION_PC2HH);
		}

	public boolean
	isActionNone()
		{
		return (this.syncAction == HOTSYNC_ACTION_NONE);
		}

	public void
	loadConfiguration()
		{
		DataInputStream dataIn = null;

		try {
			dataIn =
				new DataInputStream
					( new FileInputStream
						( this.getPropertiesFile() ) );

			this.syncAction = dataIn.readShort();

			this.loadConduitConfig( dataIn );

			dataIn.close();
			}
		catch ( IOException ex )
			{
			}
		finally
			{
			if ( dataIn != null )
				{
				try { dataIn.close(); }
				catch ( IOException ex ) {}
				}
			}
		}

	public void
	saveConfiguration()
		{
		DataOutputStream dataOut = null;
		try {
			dataOut =
				new DataOutputStream
					( new FileOutputStream
						( this.getPropertiesFile() ) );

			dataOut.writeShort( this.syncAction );

			this.saveConduitConfig( dataOut );
			}
		catch ( IOException ex )
			{
			}
		finally
			{
			if ( dataOut != null )
				{
				try { dataOut.close(); }
				catch ( IOException ex ) {}
				}
			}
		}

	}
