
package com.dotfile.palm.config;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.ImageProducer;
import java.io.*;
import java.util.*;
import java.net.*;

import javax.mail.*;
import javax.mail.internet.*;
import palm.conduit.*;

import com.dotfile.palm.awt.BorderPanel;
import com.dotfile.palm.awt.ImageIcon;
import com.dotfile.palm.config.HotSyncConfig;
import com.dotfile.palm.config.HotSyncConstants;


public
abstract
class		HotSyncDialog
extends		Dialog
implements	ActionListener, HotSyncConstants, WindowListener
	{
	/**
	 * Dialog used by Configure UI
	 */
	BorderPanel iconPan;

	Checkbox syncBtn;
	Checkbox upBtn;
	Checkbox downBtn;
	Checkbox noneBtn;

	Image syncImage;
	Image upImage;
	Image downImage;
	Image noneImage;

	/**
	 * Button used by Configure UI
	 */
	Button saveBtn;
	Button dismissBtn;

	HotSyncConfig config;


	abstract public void
		updateConfiguration();

	abstract public Panel
		layoutConfigPanel();

	public
	HotSyncDialog(
			HotSyncConfig config, Frame frm, String title, boolean modal,
			String[] btnTitles, boolean[] btnEnable )
		{
		super( frm, title, modal );

		this.config = config;

		this.layoutConfigDialog( btnTitles, btnEnable );

		// These repaints are a hack to fix a lack of refresh
		// on the very first open of the dialog. No clue why.
		//
		this.iconPan.repaint(200);
		}

	protected HotSyncConfig
	getConfig()
		{
		return this.config;
		}

	public void
	actionPerformed( ActionEvent evt )
		{
		// Button pressed, save changes
		if ( evt.getSource() == this.saveBtn )
			{
			if ( this.downBtn.getState() )
				this.config.setSyncAction
					( HOTSYNC_ACTION_PC2HH );
			else if ( this.upBtn.getState() )
				this.config.setSyncAction
					( HOTSYNC_ACTION_HH2PC );
			else if ( this.noneBtn.getState() )
				this.config.setSyncAction
					( HOTSYNC_ACTION_NONE );
			else
				this.config.setSyncAction
					( HOTSYNC_ACTION_SYNC );

			this.updateConfiguration();

			this.config.saveConfiguration();
			}

		this.setVisible( false );
		}

	private void
	layoutConfigDialog( String[] btnTitles, boolean[] btnEnable )
		{
		Dimension scrnSz =
			this.getToolkit().getScreenSize();

		this.setResizable( false );
		this.addWindowListener( this );
		this.setLayout( new BorderLayout() );

		Panel edgePan = new Panel();
		edgePan.setLayout( new GridBagLayout() );
		this.add( edgePan );

		Panel cfgPan = new Panel();
		cfgPan.setLayout( new BorderLayout() );
		this.constrain(
			edgePan, cfgPan,
			GridBagConstraints.BOTH,
			GridBagConstraints.CENTER,
			0, 0, 1, 1, 1.0, 1.0,
			new Insets( 5, 5, 5, 5 ) );

		int fldRow = 0;
		Panel fldPan = new Panel();
		fldPan.setLayout( new GridBagLayout() );

		//
		// THE SPECIFIC CONFIG PANEL
		//
		Panel subConfigPan = this.layoutConfigPanel();

		if ( subConfigPan != null )
			{
			this.constrain(
				fldPan, subConfigPan,
				GridBagConstraints.BOTH,
				GridBagConstraints.CENTER,
				0, fldRow++, 1, 1, 1.0, 1.0,
				new Insets( 1, 0, 0, 0 ) );
			}

		int row = 0;

		//
		// LOAD ICON IMAGES
		//
		MediaTracker tracker = new MediaTracker( this );
		try {
			this.syncImage =
				this.getImageResource
					( "/com/dotfile/palm/config/images/sync.gif" );
			tracker.addImage( this.syncImage, 0 );

			this.upImage =
				this.getImageResource
					( "/com/dotfile/palm/config/images/up.gif" );
			tracker.addImage( this.upImage, 0 );

			this.downImage =
				this.getImageResource
					( "/com/dotfile/palm/config/images/down.gif" );
			tracker.addImage( this.downImage, 0 );

			this.noneImage =
				this.getImageResource
					( "/com/dotfile/palm/config/images/none.gif" );
			tracker.addImage( this.noneImage, 0 );
			}
		catch ( IOException ex )
			{
			}

		try { tracker.waitForAll(); }
		catch ( InterruptedException ex )
			{
			}

		ImageIcon icon;
		CheckboxGroup chkGrp = new CheckboxGroup();

		//
		// ICONS CONFIG PANEL
		//
		this.iconPan =
			new BorderPanel
				( 5, 2, 3, BorderPanel.RIDGE, "HotSync Action" );

		iconPan.setLayout( new GridBagLayout() );
		this.constrain(
			fldPan, this.iconPan,
			GridBagConstraints.BOTH,
			GridBagConstraints.CENTER,
			0, fldRow++, 1, 1, 1.0, 1.0,
			new Insets( 5, 0, 0, 0 ) );

		row = 0;

		icon = new ImageIcon( this.syncImage );
		this.constrain(
			this.iconPan, icon,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			0, row, 1, 1, 0.0, 0.0,
			new Insets( 3, 7, 3, 7 ) );

		this.syncBtn =
			new Checkbox
				( btnTitles[0], this.config.isActionSync(), chkGrp );
		this.syncBtn.setEnabled( btnEnable[0] );
		this.constrain(
			this.iconPan, this.syncBtn,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 5, 0, 5 ) );

		icon = new ImageIcon( this.downImage );
		this.constrain(
			this.iconPan, icon,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			0, row, 1, 1, 0.0, 0.0,
			new Insets( 3, 7, 3, 7 ) );

		this.downBtn =
			new Checkbox
				( btnTitles[1], this.config.isActionPC2HH(), chkGrp );
		this.downBtn.setEnabled( btnEnable[1] );
		this.constrain(
			this.iconPan, this.downBtn,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 5, 0, 5 ) );

		icon = new ImageIcon( this.upImage );
		this.constrain(
			this.iconPan, icon,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			0, row, 1, 1, 0.0, 0.0,
			new Insets( 3, 7, 3, 7 ) );

		this.upBtn =
			new Checkbox
				( btnTitles[2], this.config.isActionHH2PC(), chkGrp );
		this.upBtn.setEnabled( btnEnable[2] );
		this.constrain(
			this.iconPan, this.upBtn,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 5, 0, 5 ) );

		icon = new ImageIcon( this.noneImage );
		this.constrain(
			this.iconPan, icon,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			0, row, 1, 1, 0.0, 0.0,
			new Insets( 3, 7, 3, 7 ) );

		this.noneBtn =
			new Checkbox
				( btnTitles[3], this.config.isActionNone(), chkGrp );
		this.noneBtn.setEnabled( btnEnable[3] );
		this.constrain(
			this.iconPan, this.noneBtn,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 5, 0, 5 ) );

		//
		// CONTROLS PANEL
		//
		Panel ctlPan = new Panel();
		ctlPan.setLayout( new GridBagLayout() );

		Canvas spacer = new Canvas();
		this.constrain(
			ctlPan, spacer,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.CENTER,
			0, 0, 1, 1, 1.0, 0.0,
			new Insets( 1, 1, 1, 1 ) );

		this.dismissBtn = new Button( "Dismiss" );
		this.dismissBtn.addActionListener( this );
		this.constrain(
			ctlPan, this.dismissBtn,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			1, 0, 1, 1, 0.0, 0.0,
			new Insets( 9, 7, 3, 7 ) );

		this.saveBtn = new Button( "Save" );
		this.saveBtn.addActionListener( this );
		this.constrain(
			ctlPan, this.saveBtn,
			GridBagConstraints.NONE,
			GridBagConstraints.CENTER,
			2, 0, 1, 1, 0.0, 0.0,
			new Insets( 9, 7, 3, 3 ) );

		//
		// PLACE PANELS IN DIALOG CONTAINER
		//
		cfgPan.add( BorderLayout.CENTER, fldPan );

		cfgPan.add( BorderLayout.SOUTH, ctlPan );

		this.pack();

		// Center it on the screen
		Dimension sz = this.getSize();
		this.setLocation
			( ((scrnSz.width - sz.width) / 2),
				((scrnSz.height - sz.height) / 2) );
		}

	public Image
	getImageResource( String name )
		throws java.io.IOException
		{
		return this.getImageResource( this.getClass(), name );
		}

	public Image
	getImageResource( Class base, String name )
		throws java.io.IOException
		{
		Image	result = null;

		URL imageURL = base.getResource( name );

		if ( imageURL != null )
			{
			Toolkit	tk = Toolkit.getDefaultToolkit();
			result = tk.createImage
				( (ImageProducer) imageURL.getContent() );
			}
		
		return result;
		}

	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e)
		{
		this.setVisible( false );
		}

	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}

	public void
	constrain(
			Container container, Component component,  
			int fill, int anchor,
			int gx, int gy, int gw, int gh, double wx, double wy )
		{
		GridBagConstraints	c =
			new GridBagConstraints();

		c.fill = fill;
		c.anchor = anchor;
		c.gridx = gx;
		c.gridy = gy;
		c.gridwidth = gw;
		c.gridheight = gh;
		c.weightx = wx;
		c.weighty = wy;

		( (GridBagLayout)container.getLayout() ).
			setConstraints( component, c );

		container.add( component );
		}

	public void
	constrain(
			Container container, Component component,  
			int fill, int anchor,
			int gx, int gy, int gw, int gh,
			double wx, double wy, Insets inset )
		{
		GridBagConstraints	c =
			new GridBagConstraints();

		c.fill = fill;
		c.anchor = anchor;
		c.gridx = gx;
		c.gridy = gy;
		c.gridwidth = gw;
		c.gridheight = gh;
		c.weightx = wx;
		c.weighty = wy;
		c.insets = inset;

		( (GridBagLayout)container.getLayout() ).
			setConstraints( component, c );

		container.add( component );
		}

	public void
	constrain(
			Container container, Component component,  
			int fill, int anchor,
			int gx, int gy, int gw, int gh,
			double wx, double wy,
			int ipadx, int ipady )
		{
		GridBagConstraints	c =
			new GridBagConstraints();

		c.fill = fill;
		c.anchor = anchor;
		c.gridx = gx;
		c.gridy = gy;
		c.gridwidth = gw;
		c.gridheight = gh;
		c.weightx = wx;
		c.weighty = wy;
		c.ipadx = ipadx;
		c.ipady = ipady;

		( (GridBagLayout)container.getLayout() ).
			setConstraints( component, c );

		container.add( component );
		}
	}

