
package com.dotfile.palm.currexc;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.ImageProducer;
import java.io.*;
import java.util.*;
import java.net.*;

import javax.mail.*;
import javax.mail.internet.*;
import palm.conduit.*;

import com.dotfile.palm.awt.BorderPanel;
import com.dotfile.palm.awt.ImageIcon;
import com.dotfile.palm.config.HotSyncConstants;
import com.dotfile.palm.config.HotSyncDialog;


public
class		ConfigDialog
extends		HotSyncDialog
implements	ItemListener
	{
	private static final String[] btnTitles =
		{
		"Synchronize the files.",
		"Update CurrencyDB from web.",
		"HandHeld overwrites PC.",
		"Do nothing."
		};

	private static final boolean[] btnEnable =
		{ false, true, false, true };

	/**
	 * Panels used in the dialog.
	 */
	BorderPanel urlPan;
	BorderPanel proxyPan;

	/**
	 * The URL of the currency update PDB.
	 */
	TextField urlField;

	/**
	 * The hostname of the proxy service
	 */
	TextField hostField;

	/**
	 * The port number of the proxy service
	 */
	TextField portField;

	/**
	 * The checkbox that controls proxy use
	 */
	Checkbox proxyCheck;

	/**
	 * The checkbox that sets the debug flag
	 */
	Checkbox debugCheck;

	Button redBtn;
	Button fullBtn;


	public
	ConfigDialog( CurrExConfig config, Frame frm, String title, boolean modal )
		{
		super( config, frm, title, modal, btnTitles, btnEnable );
		//
		// These repaints are a hack to fix a lack of refresh
		// on the very first open of the dialog. No clue why.
		//
		this.urlPan.repaint(200);
		}

	public void
	updateConfiguration()
		{
		CurrExConfig cfg = (CurrExConfig) this.getConfig();

		cfg.setDebug( this.debugCheck.getState() );

		cfg.setUpdateURL( this.urlField.getText() );

		if ( this.proxyCheck.getState() )
			{
			cfg.setUseProxy( true );
			cfg.setProxyHost( this.hostField.getText() );
			int port = 80;
			try { port = Integer.parseInt( this.portField.getText() ); }
			catch ( NumberFormatException ex ) { port = 80; }
			cfg.setProxyPort( port );
			}
		else
			{
			cfg.setUseProxy( false );
			}
		}

	public void
	actionPerformed( ActionEvent event )
		{
		super.actionPerformed( event );
		}

	public void
	itemStateChanged( ItemEvent event )
		{
		if ( event.getSource() == this.proxyCheck )
			{
			if ( this.proxyCheck.getState() )
				{
				this.hostField.setEnabled( true );
				this.portField.setEnabled( true );
				}
			else
				{
				this.hostField.setEnabled( false );
				this.portField.setEnabled( false );
				}
			}
		}

	public Panel
	layoutConfigPanel()
		{
		CurrExConfig cfg = (CurrExConfig) this.getConfig();

		this.urlPan =
			new BorderPanel( 5, 2, 3, BorderPanel.RIDGE, "Update" );

		this.urlPan.setLayout( new GridBagLayout() );

		int row = 0;

		this.constrain(
			this.urlPan, new Label( "Currency Database Update URL:" ),
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 2, 0, 2 ) );

		this.urlField = new TextField( cfg.getUpdateURL() );

		this.constrain(
			this.urlPan, this.urlField,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row++, 3, 1, 1.0, 0.0,
			new Insets( 0, 2, 0, 2 ) );

		this.proxyPan =
			new BorderPanel
				( 5, 2, 3, BorderPanel.RIDGE, "Proxy Connection" );

		this.proxyPan.setLayout( new GridBagLayout() );

		this.constrain(
			this.urlPan, this.proxyPan,
			GridBagConstraints.BOTH,
			GridBagConstraints.CENTER,
			0, row++, 3, 1, 1.0, 1.0,
			new Insets( 4, 1, 0, 1 ) );

		this.debugCheck = new Checkbox( "Log Debug Info?" );
		this.debugCheck.setState( cfg.getDebug() );

		this.constrain(
			this.urlPan, this.debugCheck,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row++, 1, 1, 1.0, 0.0,
			new Insets( 0, 5, 0, 1 ) );

		row = 0;

		this.proxyCheck = new Checkbox( "Use Proxy?" );
		this.proxyCheck.setState( cfg.getUseProxy() );
		this.proxyCheck.addItemListener( this );

		this.constrain(
			this.proxyPan, this.proxyCheck,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row++, 2, 1, 1.0, 0.0,
			new Insets( 2, 3, 0, 3 ) );

		this.constrain(
			this.proxyPan, new Label( "Proxy Hostname:" ),
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row, 1, 1, 0.85, 0.0,
			new Insets( 0, 3, 0, 2 ) );

		this.constrain(
			this.proxyPan, new Label( "Port" ),
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 0.15, 0.0,
			new Insets( 0, 3, 0, 2 ) );

		this.hostField = new TextField( cfg.getProxyHost() );

		this.constrain(
			this.proxyPan, this.hostField,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			0, row, 1, 1, 0.85, 0.0,
			new Insets( 0, 2, 4, 2 ) );

		this.portField = new TextField( "" + cfg.getProxyPort() );

		this.constrain(
			this.proxyPan, this.portField,
			GridBagConstraints.HORIZONTAL,
			GridBagConstraints.WEST,
			1, row++, 1, 1, 0.15, 0.0,
			new Insets( 0, 2, 4, 2 ) );

		if ( ! cfg.getUseProxy() )
			{
			this.hostField.setEnabled( false );
			this.portField.setEnabled( false );
			}

		return this.urlPan;
		}

	}

