package com.dotfile.palm.currexc;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;
import palm.conduit.*;


/**
 *  Simple example of a java conduit which accesses an application (.prc file) from the
 *	web and installs it on the hand-held device.
 */

public
class		CurrExConduit
implements	Conduit
	{
	public static final String		VERSION = "1.3";

	/** Dialog used by Configure UI
	 */
	ConfigDialog configDlg;
	
	/** This conduit's configuration
	 */
	CurrExConfig config;

	int syncType = SyncProperties.SYNC_DO_NOTHING;


	/**
	 * Returns a String representation of the conduit name.
	 */
	public String
	name()
		{
		return "Currency Exchange Conduit";
		}

	/**
	 * Opens and runs the sample conduit.
	 */
    public void
	open( SyncProperties props )
		{
		this.syncType = props.SYNC_DO_NOTHING;

		String status = "OK";

		Log.startSync();

		this.config = new CurrExConfig( props.pathName );

		if ( this.config.getDebug() )
			{
			Log.out( "Currency Conduit Version " + VERSION );
			Log.out( "Conduit Properties:" );
			Log.out( "  appInfoSize    = " + props.appInfoSize );
			Log.out( "  cardNo         = " + props.cardNo );
			Log.out( "  connType       = " + props.connType );
			Log.out( "  creator        = " + props.creator );
			Log.out( "  dbType         = " + props.dbType );
			Log.out( "  firstDevice    = " + props.firstDevice );
			Log.out( "  hKey           = " + props.hKey );
			Log.out( "  localName      = " + props.localName );
			Log.out( "  pathName       = " + props.pathName );
			Log.out( "  registry       = " + props.registry );
			Log.out( "  sortInfoSize   = " + props.sortInfoSize );
			Log.out( "  remoteNames    = " + props.remoteNames );
			Log.out( "  remoteNamesLEN = " + props.remoteNames.length );
			for ( int i = 0 ; i < props.remoteNames.length ; ++i )
				Log.out( "  remoteNames["+i+"] = " + props.remoteNames[i] );

			Log.out( "Conduit Configuration:" );
			Log.out( "  action         = " + this.config.getSyncAction() );
			Log.out( "  updateURL      = " + this.config.getUpdateURL() );
			Log.out( "  useProxy?      = " + this.config.getUseProxy() );
			Log.out( "  proxyHost      = " + this.config.getProxyHost() );
			Log.out( "  proxyPort      = " + this.config.getProxyPort() );
			}

		try {
			if ( this.config.isActionSync()
					|| this.config.isActionPC2HH() )
				{
				CurrExDB db =
					new CurrExDB
						( this.readURLBytes ( this.config.getUpdateURL() ),
						  false, this.config.getDebug() );

				db.writeToDevice();
				}
			}
		catch ( MalformedURLException ex )
			{
			status = "ERR";
			Log.err( "URL Error: " + ex.getMessage() );
			}
		catch ( SyncException ex )
			{
			status = "ERR";
			Log.err( "Sync error: " + ex.getMessage() );
			}
		catch( IOException ex )
			{
			status = "ERR";
			Log.err( "Read/Write Error: " + ex.getMessage() );
			}

		Log.out( status + " Currency" );

		Log.endSync();

		try {
			SyncManager.addLogEntry
				( status + " Currency\n" );
			}
		catch ( SyncException ex )
			{ }
		}
    
    /** reads the .pdb from the URL in a byte stream
    */

    DataInputStream
	readURLBytes( String urlName )
		throws IOException
		{
		int bytesRead;
		byte[] data = new byte[1024];

		if ( this.config.getUseProxy() )
			{
			Properties props = System.getProperties();
			props.put( "proxySet", "true" );
			props.put( "proxyHost", this.config.getProxyHost() );
			props.put( "proxyPort", ""+this.config.getProxyPort() );
			}
		else
			{
			Properties props = System.getProperties();
			if ( props.get( "proxySet" ) != null )
				props.remove( "proxySet" );
			if ( props.get( "proxyHost" ) != null )
				props.remove( "proxyHost" );
			if ( props.get( "proxyPort" ) != null )
				props.remove( "proxyPort" );
			}

		ByteArrayOutputStream out = new ByteArrayOutputStream();

		InputStream in =
			new URL( urlName ).openConnection().getInputStream();

		do {
			try {
				bytesRead = in.read( data );
				}
			catch ( InterruptedIOException ex )
				{
				bytesRead = ex.bytesTransferred;
				}

			if ( bytesRead > 0 )
				{
				out.write( data, 0, bytesRead );
				}

			} while ( bytesRead >= 0 );

		try {
			in.close();
			out.flush();
			out.close();
			}
		catch ( IOException ex )
			{}

		return
			new DataInputStream
				( new ByteArrayInputStream( out.toByteArray() ) );
		}

	/**
	 *  Entry point for conduit configuration through HS Manager.
	 *  Code for UI can go here or be somewhere else & kicked off
	 *  from here.
	 */

	public int
	configure( ConfigureConduitInfo info )
		{
		this.config = new CurrExConfig( info.pathName );

		Dimension scrnSz, sz;
		Frame f = new Frame();

		scrnSz = f.getToolkit().getScreenSize();

		f.setSize( 35, 20 );
		f.setLocation
			( ((scrnSz.width - 35) / 2),
				((scrnSz.height - 20) / 2) );

	//	f.show();
		f.addNotify();

		// Create a dialog
		this.configDlg =
			new ConfigDialog
				( this.config, f, "Currency Conduit " + VERSION,
				  true );

		this.configDlg.show();

		f.setVisible( false );
		f.dispose();

		return 0;
		}
	}
