
package com.dotfile.palm.currexc;

import java.io.*;
import java.util.*;
import palm.conduit.*;


public
class	CurrExDB
	{
	public static final int HDR_SIZE = 78;
	public static final int REC_SIZE = 8;
	public static final int RSRC_SIZE = 10;


    Vector	records;
	boolean debug = false;
    boolean	isPrc = false;

    int		cardNum = 0;
    int		dbFlags = 0;
    String	dbType = "";
    String	name = "";
    String	creator = "";
    int		uniqueIdSeed = 0;
    int		nextRecordList = 0;
    int		version = 0;
    int		modNumber = 0;
    int		appInfo = 0;
    int		sortInfo = 0;
    int		index = 0;
    int		createDate = 0;
    int		modDate = 0;
    int		backupDate = 0;
    int		recCount = 0;
    int		dwReserved = 0;

    static int
	strSigToInt( String s )
		{
		return
			((((int) s.charAt(0)) << 24) +
			 (((int) s.charAt(1)) << 16) +
			 (((int) s.charAt(2)) <<  8) +
			 (((int) s.charAt(3))));
		}

    static String
	intSigToStr( int i )
		{
		return
			("" +
			((char) ((i & 0xFF000000) >>> 24)) +
			((char) ((i & 0x00FF0000) >>> 16)) +
			((char) ((i & 0x0000FF00) >>>  8)) +
			((char) ((i & 0x000000FF))));
		}

    static String
	readPaddedString( DataInputStream in, int length )
		throws IOException
		{
		StringBuffer str = new StringBuffer();
		for ( ; length-- > 0 ; )
			{
			byte b = in.readByte();
			if ( b != 0 )
				{
				str.append( (char)b );
				}
			}

		return str.toString();
		}

    public
	CurrExDB( DataInputStream in, boolean isPrc, boolean debug )
		throws IOException
		{
		this.isPrc = isPrc;
		this.debug = debug;
		this.loadDB( in );
		}

    public void
	writeToDevice()
		throws IOException
		{
		CurrExRecord rec;
		int filehandle = 0;

		try {
			filehandle =
				SyncManager.openDB
					( name, 0,
						(SyncManager.OPEN_READ |
						 SyncManager.OPEN_WRITE |
						 SyncManager.OPEN_EXCLUSIVE) );

			SyncManager.closeDB( filehandle );
			SyncManager.deleteDB( name, 0 );
			}
		catch ( IOException ex )
			{
			if ( filehandle != 0 )
				{
				SyncManager.closeDB( filehandle );
				}
			}

		if ( isPrc )
			{
			this.dbFlags = SyncManager.DB_RESOURCE;
			}
		else
			{
			this.dbFlags = SyncManager.DB_RECORD;
			}
	
		filehandle =
			SyncManager.createDB
				( CurrExDB.strSigToInt( creator ),
					this.dbFlags, cardNum, name,
					CurrExDB.strSigToInt( dbType ) );

		int i = 0;

		if ( ! isPrc )
			{
			if ( this.appInfo != 0 )
				{
				rec = (CurrExRecord) records.elementAt( i++ );

				SyncManager.writeDBAppInfoBlock
					( filehandle, this.name, rec.getBytes() );
				}

/* UNDONE REVIEW - Don't we need this also?!
			if ( this.sortInfo != 0 )
				{
				rec = (CurrExRecord) records.elementAt( i++ );

				SyncManager.writeDBSortInfoBlock
					( filehandle, this.name, rec.getBytes() );
				}
*/
			}

		if ( this.debug )
			Log.out( "CurrEx: Write Loop i = "
					+ i + ", size = " + this.records.size() );

		for ( ; i < this.records.size() ; i++ )
			{
			rec = (CurrExRecord) this.records.elementAt( i );

			if ( isPrc )
				{
				SyncManager.writeResourceRec(filehandle, rec);
				}
			else
				{
				SyncManager.writeRec(filehandle, rec);
				}
			}

		SyncManager.closeDB(filehandle);
		}

    protected Vector
	readPRCHeaders( DataInputStream in )
		throws IOException
		{
		Vector recs = new Vector( this.recCount );

		for ( int i = 0 ; i < this.recCount ; i++ )
			{
			CurrExRecord rec =
				new CurrExRecord
					( in.readInt(), in.readShort(), in.readInt() );

			recs.addElement( rec );
			}

		return recs;
		}


    protected Vector
	readPDBHeaders( DataInputStream in )
		throws IOException
		{
		Vector recs = new Vector( recCount );

		if ( this.debug )
			Log.out( "CurrEx: this.appInfo = " + this.appInfo );

		if ( this.appInfo != 0 )
			{
			CurrExRecord rec =
				new CurrExRecord( this.appInfo, 0 );

			recs.addElement( rec );
			}

/* UNDONE REVIEW - Don't we need this also?!
		if ( this.sortInfo != 0 )
			{
			CurrExRecord rec =
				new CurrExRecord( this.sortInfo, 0 );

			recs.addElement( rec );
			}
*/
		for ( int i = 0 ; i < this.recCount ; i++ )
			{
			CurrExRecord rec =
				new CurrExRecord( in.readInt(), in.readInt() );

			recs.addElement( rec );
			}

		return recs;
		}

    /**
	 *   This function reads in all the resources (.prc & .pdb) contained
     *   in the app being downloaded. It places each resource in it's own
     *   Rec object.
     */

    protected void
	loadDB( DataInputStream in )
		throws IOException
		{
		CurrExRecord rec, nextRec;

		this.name			= this.readPaddedString( in, 32 );
		this.dbFlags		= in.readShort();
		this.version		= in.readShort();
		this.createDate		= in.readInt();
		this.modDate		= in.readInt();
		this.backupDate		= in.readInt();
		this.modNumber		= in.readInt();
		this.appInfo		= in.readInt();
		this.sortInfo		= in.readInt();
		this.dbType			= this.readPaddedString( in, 4 );
		this.creator		= this.readPaddedString( in, 4 );
		this.uniqueIdSeed	= in.readInt();
		this.nextRecordList	= in.readInt();
		this.recCount		= in.readShort();

		if ( this.debug )
			{
			Log.out( "CurrEx: name      = '" + this.name + "'" );
			Log.out( "CurrEx: version   = '" + this.version + "'" );
			Log.out( "CurrEx: modNumber = '" + this.modNumber + "'" );
			Log.out( "CurrEx: recCount  = '" + this.recCount + "'" );
			Log.out( "CurrEx: dbFlags   = '" + this.dbFlags + "'" );
			Log.out( "CurrEx: dbType    = '" + this.dbType + "'" );
			Log.out( "CurrEx: creator   = '" + this.creator + "'" );
			Log.out( "CurrEx: appInfo   = '" + this.appInfo + "'" );
			Log.out( "CurrEx: sortInfo  = '" + this.sortInfo + "'" );
			}

		if ( this.isPrc )
			{
			this.records = this.readPRCHeaders( in );
			}
		else
			{
			this.records = this.readPDBHeaders( in );
			}

		int pad = 0;
		if ( this.records.size() > 0 )
			{
			rec = (CurrExRecord) this.records.elementAt( 0 );
			pad = rec.offset -
					( HDR_SIZE + ( REC_SIZE * this.recCount ) );
			if ( this.debug )
				Log.out( "CurrEx: Rec0.off  = '" + rec.offset + "'" );
			}

		if ( this.debug )
			Log.out( "CurrEx: PAD = '" + pad + "'" );

		for ( ; pad > 0 ; pad -= 2 )
			{
			in.readShort();
			}

		int cnt = this.records.size();
		for ( int i = 0; i < (cnt - 1); i++ )
			{
			rec = (CurrExRecord) this.records.elementAt( i );
			nextRec = (CurrExRecord) this.records.elementAt( i + 1 );
			rec.size = nextRec.offset - rec.offset;
			rec.readData( in );
			}

		rec = (CurrExRecord) this.records.elementAt( cnt - 1 );
		rec.size = in.available();
		if ( this.debug )
			Log.out( "CurrEx: LAST rec.size (available) = " + rec.size );
		rec.readData( in );
		}

	}
